#!/usr/bin/env python
"""
Created on 2020/04/17

@author: yyang
"""
import os
import math
import fcntl
from ctypes import c_uint8, c_uint16, c_uint32, c_uint64, Structure, Union, sizeof, addressof


class OPCODE(object):
    # Opcodes for Admin Commands
    DELETE_IO_SQ = 0x00
    CREATE_IO_SQ = 0x01
    GET_LOG_PAGE = 0x02
    DELETE_IO_CQ = 0x04
    CREATE_IO_CQ = 0x05
    IDENTIFY = 0x06
    ABORT = 0x08
    SET_FEATURE = 0x09
    GET_FEATURE = 0x0A
    ASYNC_EVENT = 0x0C
    NAMESPACE_MGT = 0x0D
    FIRMWARE_COMMIT = 0x10
    FIRMWARE_DOWNLOAD = 0x11
    NAMESPACE_ATTACH = 0x15
    FORMAT_NVM = 0x80
    SECURITY_SEND = 0x81
    SECURITY_RECV = 0x82
    # Opcodes for NVM Commands
    FLUSH = 0x00
    WRITE = 0x01
    READ = 0x02
    WRITE_UNCORRECTABLE = 0x04
    COMPARE = 0x05
    WRITE_ZEROS = 0x08
    DATASET_MGT = 0x09
    RESERVATION_REGISTER = 0x0D
    RESERVATION_REPORT = 0x0E
    RESERVATION_ACQUIRE = 0x11
    RESERVATION_RELEASE = 0x12
    # Opcodes for CNEX Commands
    CNEX_WRITE_MEM = 0x99
    CNEX_READ_MEM = 0x9A


class NVMePassthruCmd(Union):
    """
    Take Care: different with SQE!
    cdw8 is metadata_len and cdw9 is data_len
    Add cdw16: timeout
    Add cdw17: dword0 of CQE
    """

    class Bits(Structure):
        _pack_ = 1
        _fields_ = [
            ('opcode', c_uint8),
            ('flags', c_uint8),
            ('rsvd1', c_uint16),
            ('nsid', c_uint32),
            ('cdw2', c_uint32),
            ('cdw3', c_uint32),
            ('metadata', c_uint64),
            ('addr', c_uint64),
            ('metadata_len', c_uint32),
            ('data_len', c_uint32),
            ('lba', c_uint64),
            ('cdw12', c_uint32),
            ('cdw13', c_uint32),
            ('cdw14', c_uint32),
            ('cdw15', c_uint32),
            ('timeout_ms', c_uint32),
            ('result', c_uint32),
        ]

    class Dword(Structure):
        _pack_ = 1
        _fields_ = [
            ('dword0', c_uint32),
            ('dword1', c_uint32),
            ('dword2', c_uint32),
            ('dword3', c_uint32),
            ('dword4', c_uint32),
            ('dword5', c_uint32),
            ('dword6', c_uint32),
            ('dword7', c_uint32),
            ('dword8', c_uint32),
            ('dword9', c_uint32),
            ('dword10', c_uint32),
            ('dword11', c_uint32),
            ('dword12', c_uint32),
            ('dword13', c_uint32),
            ('dword14', c_uint32),
            ('dword15', c_uint32),
            ('dword16', c_uint32),
            ('dword17', c_uint32),
        ]

    _anonymous_ = ('dword', 'bits')
    _fields_ = [
        ('dword', Dword),
        ('bits', Bits),
    ]


def _IOWR(_tp, _nr, _sz):
    return (3 << 30) + (sizeof(_sz) << 16) + (ord(_tp) << 8) + _nr


NVME_IOCTL_ADMIN_CMD = _IOWR('N', 0x41, NVMePassthruCmd)
NVME_IOCTL_IO_CMD = _IOWR('N', 0x43, NVMePassthruCmd)


class NVMe(object):
    def __init__(self, device='/dev/nvme0', nsid=1):
        self._fd = -1
        self.nsid = nsid
        assert os.path.exists(device), 'Cannot find device: {}'.format(device)

        self._fd = os.open(device, os.O_RDWR)
        assert self._fd != -1, 'Cannot open device: {}'.format(device)

    def __del__(self):
        if self._fd != -1:
            os.close(self._fd)
            self._fd = -1

    def close(self):
        self.__del__()

    def _ioctl(self, req, cmd):
        return fcntl.ioctl(self._fd, req, cmd)

    def io_passthru(self, cmd):
        """
        Send a user-defined IO command to the specified device via IOCTL passthrough
        :param cmd:
        :return:
        """
        return self._ioctl(NVME_IOCTL_IO_CMD, cmd)

    def admin_passthru(self, cmd):
        """
        Send a user-defined Admin command to the specified device via IOCTL passthrough
        :param cmd:
        :return:
        """
        return self._ioctl(NVME_IOCTL_ADMIN_CMD, cmd)

    def rdma_read_list(self, offset, ndw=1, nsid=1):
        """
        RDMA read, return list of uint32
        :param offset:  Offset address
        :param ndw:     Number of dword
        :param nsid:    Namespace ID
        :return:        List of uint32 from RDMA read
        """
        length = math.ceil(ndw / 1024) * 1024
        data = (c_uint32 * length)()

        cmd = NVMePassthruCmd()
        cmd.opcode = OPCODE.CNEX_READ_MEM
        cmd.lba = offset
        cmd.nsid = nsid
        cmd.dword12 = ndw - 1
        cmd.addr = addressof(data)
        cmd.data_len = length * 4

        status = self.io_passthru(cmd)
        assert status == 0, 'RDMA read failed! Offset: {:#x} Status: {:#x}'.format(offset, status)

        return data[0:ndw]

    def rdma_write_list(self, offset, array, nsid=1):
        """
        RDMA write list of uint32 to offset
        :param offset:  Offset address
        :param array:   List of uint32 to RDMA write
        :param nsid:    Namespace ID
        :return:
        """
        ndw = len(array)
        length = math.ceil(ndw / 1024) * 1024
        data = (c_uint32 * length)(*array)

        cmd = NVMePassthruCmd()
        cmd.opcode = OPCODE.CNEX_WRITE_MEM
        cmd.lba = offset
        cmd.nsid = nsid
        cmd.dword12 = ndw - 1
        cmd.addr = addressof(data)
        cmd.data_len = length * 4

        status = self.io_passthru(cmd)
        assert status == 0, 'RDMA write failed! Offset: {:#x} Status: {:#x}'.format(offset, status)

    def read_uint32(self, offset):
        """
        RDMA read, return uint32
        :param offset:  Offset address
        :param nsid:    Namespace ID
        :return:        uint32
        """
        print("RDMA WRITE")
        return self.rdma_read_list(offset, ndw=1, nsid=self.nsid)[0]

    def write_uint32(self, offset, value):
        """
        RDMA write uint32 to offset
        :param offset:  Offset address
        :param value:   uint32 value to RDMA write
        :param nsid:    Namespace ID
        :return:
        """
        print("RDMA READ")
        self.rdma_write_list(offset, [value], nsid=self.nsid)



import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__)))
from .lib.executor import Executor


def run(rout, dfx_file, log_path):
    """

    :param rout: read, write register rout
    :param dfx_file: dfx file path
    :param log_path: where save log faile
    :return:
    result: result file
    pass_count: count of passed test
    fail_count: count of failed test
    """
    executor = Executor(rout)
    result, pass_count, fail_count = executor.execute(dfx_file, log_path)
    return result, pass_count, fail_count



import re
from .state import RegOperation



class DfxParser(object):

    def __init__(self):
        self.handler_func_map = {"com": self._parse_com,
                                 "var": self._parse_var_,
                                 "exp": self._parse_exp_}

    def run(self, dfx_file):
        parse_dict = dict()
        data_dict = self.read_and_splite_file(dfx_file)
        for data in data_dict:
            parse_dict[data[0]] = self.handler_func_map[data[0]](data[1])
        return parse_dict

    @staticmethod
    def read_and_splite_file(dfx_file):
        with open(dfx_file, 'rb') as f:
            content_raw = f.read()
        pat = '\[(.*?)\]:([\s\S]*?)(?=(\[.*\]:|$$))'
        data_dict = re.findall(pat, content_raw.decode())
        return data_dict

    def _parse_com(self, com):
        rets = re.findall("des.*?(\w+).*?addr.*?(\w+).*?count.*?(\w+)", com, re.DOTALL)
        return rets

    @staticmethod
    def _re_parse_write_string(line):
        temp = dict()
        rets = re.findall("reg\((.+?)\)\s*\=\s*(.*)", line)
        if rets:
            temp["reg_address"] = rets[0][0]
            temp["value"] = rets[0][1]
            temp["type"] = RegOperation.write
        else:
            pass
        return temp

    @staticmethod
    def _re_parse_read_string(line):
        temp = dict()
        rets = re.findall("(\w+)\s*\=\s*(.*)", line)
        if rets:
            temp["name"] = rets[0][0]
            temp["expression"] = rets[0][1]
            temp["type"] = RegOperation.read
        else:
            pass
        return temp

    def _parse_var_(self, var):
        var_list = list()
        lines = [each_raw.strip() for each_raw in var.split("\n") if each_raw.strip()]
        for line in lines:
            if self._comment(line) is True:
                continue
            ret = re.findall("reg\(.+?\=", line)
            if ret:
                value = self._re_parse_write_string(line)
            else:
                value = self._re_parse_read_string(line)
            var_list.append(value)
        return var_list

    @staticmethod
    def _re_parse_exp_info(line):
        exp, info = None, None
        ret = re.findall("\s*(.+?)(\{.*)", line)
        if ret:
            exp = ret[0][0]
            info = ret[0][1]
        return exp, info

    @staticmethod
    def _re_parse_equation(line):
        equation = None
        ret = re.findall("if\s*(.*?)\:", line)
        if ret:
            equation = ret[0]
        return equation

    def _parse_exp_(self, exp):
        exp_list, index = list(), 0
        lines = [item.strip() for item in exp.split("\n") if item.strip()]
        while index < len(lines):
            temp_dict = {"exp":list(), "info":list(), "equation":None}
            line = lines[index]
            if self._comment(line):
                index = index + 1
                continue
            if "if " in line:
                true_exp, true_info = self._re_parse_exp_info(lines[index+1])
                temp_dict["equation"] = self._re_parse_equation(line)
                temp_dict["exp"].append(true_exp)
                temp_dict["info"].append(true_info)
                if "else" in lines[index+2]:
                    false_exp, false_info = self._re_parse_exp_info(lines[index+3])
                    temp_dict["exp"].append(false_exp)
                    temp_dict["info"].append(false_info)
                    index = index + 4
                else:
                    index = index + 2
            else:
                exp, info = self._re_parse_exp_info(line)
                temp_dict["exp"].append(exp)
                temp_dict["info"].append(info)
                index = index + 1
            exp_list.append(temp_dict)
        return exp_list

    @staticmethod
    def _comment(each):
        if not each.startswith("#") and not each.startswith("//"):
            return False
        else:
            return True

    # def _parse_var(self, var):
    #     var_dict = OrderedDict()
    #     right_dict = {"right_string": "",
    #                   "reg_list": [],
    #                   "var_list": [],
    #                   "info": "REG_MESSAGE",
    #                   "value": 0
    #                   }
    #     left_pat = "(.*?)[ =]"
    #     right_pat = "[ =]+(.*?)(?=\{|$)"
    #     right_reg_pat = "(reg\(.*?\).*?)(?=[ \+\*\-\/\|\&\{\)]|$)"
    #     right_var_pat = "var\((\w+)\)"
    #     left_reg_pat = "reg\((.*?)\)"
    #     info_pat = "\{(.*?)\}"
    #     for each in [each_raw.strip() for each_raw in var.split("\n") if each_raw.strip()]:
    #         if not self._comment(each):
    #             tmp_right_dict = copy.deepcopy(right_dict)
    #             left_string = re.findall(left_pat, each)[0]
    #             right_string = re.findall(right_pat, each)[0]
    #             tmp_right_dict["right_string"] = right_string
    #             if "reg(" in left_string:
    #                 left_string = re.findall(left_reg_pat, left_string)[0]
    #             else:
    #                 for reg in re.findall(right_reg_pat, each):
    #                     tmp_right_dict["reg_list"].append(self._parse_reg(reg))
    #                 for var in re.findall(right_var_pat, each):
    #                     tmp_right_dict["var_list"].append(var)
    #             if "{" in each:
    #                 tmp_right_dict["info"] = re.findall(info_pat, each)[0]
    #             var_dict[left_string] = tmp_right_dict
    #     return var_dict
    #
    # def _parse_reg(self, reg):
    #     reg_dict = {"reg_string": reg,
    #                 "addr": "",
    #                 "range": "",
    #                 "value": 0
    #     }
    #     addr_pat = "reg\((.*?)\)"
    #     range_pat = "\[(.*?)\]"
    #     reg_dict["addr"] = re.findall(addr_pat, reg)[0]
    #     if "[" in reg:
    #         reg_dict["range"] = re.findall(range_pat, reg)[0]
    #     return reg_dict
    #
    # def _parse_exp(self, exp):
    #     exp_list = []
    #     each_dict = {}
    #     exp_pat = "(.*?)[\{]+"
    #     info_pat = "\{(.*?)\}"
    #     bak_dict_raw = {"dict": {"exp": [],
    #                              "info": [],
    #                              "equation": []},
    #                     "flag": 0}
    #     bak_dict = copy.deepcopy(bak_dict_raw)
    #     equa_pat = "[el]?if(.*):$"
    #     for each in [each_raw.strip() for each_raw in exp.split("\n") if each_raw.strip()]:
    #         if self._comment(each):
    #             continue
    #         elif "if" in each or "else" in each or bak_dict["flag"] != 0:
    #             if bak_dict["flag"] == 0 and "else" in each:
    #                 bak_dict["dict"]["equation"].append("True")
    #                 bak_dict["flag"] = 2
    #             elif bak_dict["flag"] == 1:
    #                 bak_dict["dict"]["exp"].append(re.findall(exp_pat, each)[0].strip())
    #                 bak_dict["dict"]["info"].append(re.findall(info_pat, each))
    #                 bak_dict["flag"] = 0
    #             elif bak_dict["flag"] == 2:
    #                 bak_dict["dict"]["exp"].append(re.findall(exp_pat, each)[0].strip())
    #                 bak_dict["dict"]["info"].append(re.findall(info_pat, each))
    #                 exp_list.append(bak_dict["dict"])
    #                 del bak_dict
    #                 bak_dict = copy.deepcopy(bak_dict_raw)
    #             else:
    #                 bak_dict["dict"]["equation"].append(re.findall(equa_pat, each)[0].strip())
    #                 bak_dict["flag"] = 1
    #         else:
    #             tmp_each_dict = copy.deepcopy(each_dict)
    #             tmp_each_dict["exp"] = re.findall(exp_pat, each)[0].strip()
    #             tmp_each_dict["info"] = re.findall(info_pat, each)
    #             exp_list.append(tmp_each_dict)
    #     #for dict in exp_list:
    #     #    print(dict)
    #     return exp_list




if __name__ == '__main__':
    text = r"D:\Automation\self_build_tools\Venus_Tool\script\dfx\nvc.dfx"
    dfx = DfxParser()
    dfx.run(text)
    # dfx.start(text)


import time
import os


class Dump(object):

    def __init__(self, dump_file=None):
        if dump_file is None:
            file_name = "dump_file_{}.log".format(self.get_time_stamp())
            self.file_name = os.path.join(os.getcwd(), file_name)
        else:
            if os.path.isdir(dump_file):
                file_name = "dump_file_{}.log".format(self.get_time_stamp())
                self.file_name = os.path.join(dump_file, file_name)
            else:
                self.file_name = dump_file
        print("Dump file: {}".format(self.file_name))
        self.file_handler = open(self.file_name, "a")

    def __del__(self):
        self.file_handler.close()

    def get_result_path(self):
        return self.file_name

    @staticmethod
    def get_time_stamp():
        return time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime(time.time()))

    def log(self, log_content):
        self.file_handler.write(log_content)

    def dump_com(self, name, register, index, value):
        name = self.format_(name, 20)
        register = self.format_(register, 15)
        index = self.format_(index, 10)
        value = self.format_(value, 10)
        context = "{} {} {} {}\n".format(name, register, index, value)
        self.file_handler.write(context)

    def dump(self, name=None, operation=None, value=None, expression=None, ):
        name, operation, value, expression = self.format_log(name, operation, value, expression)
        context = "{} {} {} {}\n".format(name, operation, value, expression)
        self.file_handler.write(context)

    def format_log(self, name, operation, value, expression):
        name = self.format_(name, 30)
        operation = self.format_(operation, 10)
        expression = self.format_(expression, 50)
        value = self.format_(value, 15)
        return name, operation, value, expression

    def format_(self, string_, max_len):
        if type(string_) == int:
            temp = "0x{:x}".format(string_)
            new_string = "{}{}".format(temp, " "*(max_len-len(temp)))
        else:
            if string_ is not None:
                max_len = len(string_) if len(string_) > max_len else max_len
                new_string = "{}{}".format(string_, " " * (max_len - len(string_)))
            else:
                new_string = " "*max_len
        return new_string



#!/usr/bin/env python
"""
Created on 2020/04/18

@author: yyang
"""
import os
import mmap


class PCIe(object):
    def __init__(self, filename='/sys/bus/pci/devices/0000:01:00.0/resource0'):
        self._file = None
        self._mmap = None

        assert os.path.exists(filename), 'Cannot find filename: {}'.format(filename)

        self._file = open(filename, 'wb+')
        assert self._file, 'Cannot open filename: {}'.format(filename)

        self._mmap = mmap.mmap(self._file.fileno(), 0x0)

    def __del__(self):
        if self._mmap:
            self._mmap.close()
            self._mmap = None

        if self._file:
            self._file.close()
            self._file = None

    def close(self):
        self.__del__()

    def read_uint8(self, offset):
        """
        Read PCIe bar uint8
        :param offset:  Offset address
        :return:        uint8 value that read
        """
        self._mmap.seek(offset)
        return int.from_bytes(self._mmap.read(1), byteorder='little', signed=False)

    def read_uint32(self, offset):
        """
        Read PCIe bar uint32
        :param offset:  Offset address
        :return:        uint32 value that read
        """
        self._mmap.seek(offset)
        print("PCIE READ")
        return int.from_bytes(self._mmap.read(4), byteorder='little', signed=False)

    def write_uint8(self, offset, value):
        """
        Write PCIe bar uint8
        :param offset:  Offset address
        :param value:   uint8 value to write
        :return:
        """
        self._mmap.seek(offset)
        self._mmap.write(value.to_bytes(length=1, byteorder='little', signed=False))

    def write_uint32(self, offset, value):
        """
        Write PCIe bar uint32
        :param offset:  Offset address
        :param value:   uint32 value to write
        :return:
        """
        self._mmap.seek(offset)
        print("PCIE WRITE")
        self._mmap.write(value.to_bytes(length=4, byteorder='little', signed=False))



import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__)))
import argparse
from .lib.executor import Executor
from .lib.driver.nvme import NVMe
from .lib.driver.pcie import PCIe


def create_parser():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    add_rdma_argument(subparsers)
    add_pcie_argument(subparsers)
    add_list_argument(subparsers)
    return parser


def add_list_argument(subparsers):
    regression_parser = subparsers.add_parser("list", help='list all dfx files')
    regression_parser.set_defaults(executor_function=list_handler)


def add_pcie_argument(subparsers):
    regression_parser = subparsers.add_parser("pcie", help='read and write register with PCIE')
    regression_required_arguments = regression_parser.add_argument_group('required arguments')
    regression_required_arguments.add_argument('--dfx_file', '-f', type=str, help='dfx file', required=False)
    regression_required_arguments.add_argument('--pcie', '-p', type=str, default="0000:01:00.0",
                                               help='pcie address', required=False)
    regression_parser.set_defaults(executor_function=pcie_handler)


def add_rdma_argument(subparsers):
    regression_parser = subparsers.add_parser("rdma", help='read and write register with RDMA')
    regression_required_arguments = regression_parser.add_argument_group('required arguments')
    regression_required_arguments.add_argument('--dfx_file', '-f', type=str, help='dfx file', required=False)
    regression_required_arguments.add_argument('--device', '-d', type=str,  default="nvme0",
                                               help='nvme device name', required=False)
    regression_required_arguments.add_argument('--nsid', '-n', type=int, default=1,
                                               help='nvme namespace id', required=False)
    regression_parser.set_defaults(executor_function=rdma_handler)


def rdma_handler(args):
    device = "/dev/{}".format(args.device)
    rout = NVMe(device, args.nsid)
    executor = Executor(rout)
    executor.execute(args.dfx_file)
    return 0


def pcie_handler(args):
    filename = '/sys/bus/pci/devices/{}/resource0'.format(args.pcie)
    rout = PCIe(filename)
    executor = Executor(rout)
    executor.execute(args.dfx_file)
    return 0


def list_handler(args):
    dfx_folder = os.path.join(os.path.dirname(__file__), "configuration")
    all_files = os.listdir(dfx_folder)
    for item in all_files:
        if "dfx" in item:
            print(item)


def main():
    parser = create_parser()
    args = parser.parse_args()
    try:
        args.executor_function(args)
    except Exception as e:
        print(e)

if __name__ == '__main__':
    main()




import re
import os
from .parsing import DfxParser
from .state import RegOperation
from .dump import Dump


class Executor(object):

    def __init__(self, rout):
        self.parser = DfxParser()
        self.log_dump = None
        self.rout = rout
        self.variable_list = list()
        self.dfx_folder = os.path.join(os.path.dirname(__file__), "..", "configuration")
        self.pass_count = 0
        self.fail_count = 0

    def execute(self, dfx_file, log_folder=None):
        full_path = self.get_full_path(dfx_file)
        if full_path is not None:
            self.log_dump = Dump(log_folder)
            parse_dict = self.parser.run(full_path)
            self.print_coms(parse_dict["com"])
            self.read_and_write_reg(parse_dict["var"])
            self.check_expected(parse_dict["exp"])
        return self.log_dump.get_result_path(), self.pass_count, self.fail_count

    def print_coms(self, coms):
        self.log_dump.log("[COM]\n")
        self.log_dump.dump_com("Name", "Register", "Index", "Value")
        for item in coms:
            for index in range(int(item[2])):
                reg_addr = int(item[1], 16)
                reg_addr = reg_addr + index*4
                value = self.rout.read_uint32(reg_addr)
                self.log_dump.dump_com(item[0], "{:#x}".format(reg_addr), str(index), "{:#x}".format(value))

    def check_expected(self, expressions):
        self.log_dump.log("\n[EXP]\n")
        for expression in expressions:
            if expression["equation"] is not None:
                if self.calc_expression(expression["equation"]):
                    expression_string = expression["exp"][0]
                    expression_info = expression["info"][0]
                else:
                    if len(expression["exp"])>1:
                        expression_string = expression["exp"][1]
                        expression_info = expression["info"][1]
                    else:
                        expression_string, expression_info = None, None
            else:
                expression_string = expression["exp"][0]
                expression_info = expression["info"][0]
            if expression_string is not None:
                result, convert_exp = self.calc_expression(expression_string)
                if result:
                    self.pass_count += 1
                    self.log_dump.log("Pass   {}   {} => {}\n".format(expression_info, expression_string, convert_exp))
                else:
                    self.fail_count += 1
                    self.log_dump.log("Fail   {}   {} => {}\n".format(expression_info, expression_string, convert_exp))

    def calc_expression(self, expression):
        convert_exp = expression
        for item in self.variable_list:
            if item["name"] in convert_exp:
                convert_exp = convert_exp.replace(item["name"], "0x%x" % item["value"])
        try:
            result = eval(convert_exp)
        except Exception as e:
            print(e)
            result = False
        return result, convert_exp

    def get_full_path(self, dfx_file):
        if os.path.exists(dfx_file) is False:
            full_path = os.path.join(self.dfx_folder, dfx_file)
            if os.path.exists(full_path) is False:
                full_path = os.path.join(os.getcwd(), dfx_file)
                if os.path.exists(full_path) is False:
                    print("{} not exist".format(dfx_file))
                    full_path = None
        else:
            full_path = dfx_file
        return full_path

    def read_and_write_reg(self, vars):
        self.log_dump.log("\n[var]\n")
        self.log_dump.dump("Name", "Operation", "Value", "Expression")
        for var in vars:
            if var["type"] == RegOperation.write:
                self.write_reg(var["reg_address"], var["value"])
            else:
                self.read_var(var)

    def read_var(self, var):
        expression = self.get_read_var(var["expression"])
        if expression is None:
            print("expression failed; {}".format(expression))
        else:
            expression = self.get_expression_var(expression)
            if expression is None:
                print("expression failed: {}".format(expression))
            else:
                value = eval(expression)
                final_value = {
                    "name": var["name"],
                    "value": value
                }
                self.variable_list.append(final_value)
                self.log_dump.dump(var["name"], "read", value, var["expression"])

    def get_read_var(self, expression):
        reg_list = re.findall("(reg.*?\])", expression)
        if reg_list:
            for item in reg_list:
                value = self.get_expression_reg(item)
                if value is None:
                    return None
                expression = expression.replace(item, str(value))
        return expression

    def get_expression_var(self, expression):
        vars = re.findall("var\(\w+\)", expression)
        if vars:
            for item in vars:
                var_name = re.findall("var\((\w+)\)", item)[0]
                get_value = [var["value"] for var in self.variable_list if var["name"] == var_name]
                if get_value:
                    expression = expression.replace(item, str(get_value[0]))
                else:
                    return None
        return expression

    def get_expression_reg(self, reg_string):
        value = None
        rets = re.findall("reg\((\w+)\)\[(.+)\]", reg_string)
        if rets:
            reg_addr = int(rets[0][0], 16)
            reg_range = rets[0][1]
            orgi_value = self.rout.read_uint32(reg_addr)
            value = self.get_value_with_range(orgi_value, reg_range)
        return value

    def write_reg(self, offset, value):
        get_value = eval(self.get_expression_var(value))
        offset = int(offset, 16)
        self.rout.write_uint32(offset, get_value)
        self.log_dump.dump(operation="write", value=get_value, expression="Register 0x{:x}".format(offset))

    @staticmethod
    def get_value_with_range(value, bit_range):
        value_ = None
        rets = re.findall("(\d+)\:(\d+)", bit_range)
        if rets:
            start_ = int(rets[0][1])
            end_ = int(rets[0][0])
            value_ = (value&(2**(end_+1)-1)) >> start_
        return value_


if __name__ == '__main__':
    E = Executor("pcie")
    E.execute("nsc_read.dfx")



from setuptools import setup, find_packages
#packages = ["cnex_dfx"]
#python setup.py sdist upload  

setup(
    name = 'cnex_dfx',
    version = '2.2.6',
    keywords = ['cnexlabs', 'dfx', 'dump'],
    description = 'cnexlabs tool for dump dfx files',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
      ],
    license = 'MIT License',
	url = 'https://pypi.org/project/cnex_dfx',
    #install_requires = [],
    packages = find_packages(),
    include_package_data=True, 
    author = 'yuwen123441',
    author_email = 'yuwen123441@qq.com',
    platforms = 'any',
    entry_points = {
        'console_scripts': [
        'cnex_dfx = cnex_dfx.run:main'
        ]}
)